@@ -19,7 +19,7 @@
 
 #include "copy.h"
 
-#include "akonadiconnection.h"
+#include "connection.h"
 #include "handlerhelper.h"
 
 #include "storage/datastore.h"
@@ -33,6 +33,7 @@
 #include "imapstreamparser.h"
 
 using namespace Akonadi;
+using namespace Akonadi::Server;
 
 bool Copy::copyItem( const PimItem &item, const Collection &target )
 {
@@ -49,7 +50,7 @@ bool Copy::copyItem( const PimItem &item, const Collection &target )
   Part::List parts;
   Q_FOREACH ( const Part &part, item.parts() ) {
     Part newPart( part );
-    newPart.setData( PartHelper::translateData( newPart.data(), part.external() )  );
+    newPart.setData( PartHelper::translateData( newPart.data(), part.external() ) );
     newPart.setPimItemId( -1 );
     parts << newPart;
   }
