@@ -20,7 +20,7 @@
 #include "fetchhelper.h"
 
 #include "akdebug.h"
-#include "akonadiconnection.h"
+#include "connection.h"
 #include "handler.h"
 #include "handlerhelper.h"
 #include "imapstreamparser.h"
@@ -32,6 +32,7 @@
 #include "storage/itemretrievalmanager.h"
 #include "storage/itemretrievalrequest.h"
 #include "storage/parthelper.h"
+#include <storage/parttypehelper.h>
 #include "storage/transaction.h"
 #include "utils.h"
 #include "intervalcheck.h"
@@ -46,36 +47,21 @@
 #include <QtSql/QSqlQuery>
 
 using namespace Akonadi;
+using namespace Akonadi::Server;
 
-FetchHelper::FetchHelper( AkonadiConnection *connection, const Scope &scope )
+FetchHelper::FetchHelper( Connection *connection, const Scope &scope, const FetchScope &fetchScope )
   : mStreamParser( 0 )
   , mConnection( connection )
   , mScope( scope )
-  , mAncestorDepth( 0 )
-  , mCacheOnly( false )
-  , mCheckCachedPayloadPartsOnly( false )
-  , mFullPayload( false )
-  , mAllAttrs( false )
-  , mSizeRequested( false )
-  , mMTimeRequested( false )
-  , mExternalPayloadSupported( false )
-  , mRemoteRevisionRequested( false )
-  , mIgnoreErrors( false )
-  , mFlagsRequested( false )
-  , mRemoteIdRequested( false )
-  , mGidRequested( false )
+  , mFetchScope( fetchScope )
 {
   std::fill( mItemQueryColumnMap, mItemQueryColumnMap + ItemQueryColumnCount, -1 );
 }
 
-void FetchHelper::setStreamParser( ImapStreamParser *parser )
-{
-  mStreamParser = parser;
-}
-
 enum PartQueryColumns {
   PartQueryPimIdColumn,
-  PartQueryNameColumn,
+  PartQueryTypeNamespaceColumn,
+  PartQueryTypeNameColumn,
   PartQueryDataColumn,
   PartQueryExternalColumn,
   PartQueryVersionColumn
@@ -87,8 +73,10 @@ QSqlQuery FetchHelper::buildPartQuery( const QVector<QByteArray> &partList, bool
   QueryBuilder partQuery( PimItem::tableName() );
   if ( !partList.isEmpty() || allPayload || allAttrs ) {
     partQuery.addJoin( QueryBuilder::InnerJoin, Part::tableName(), PimItem::idFullColumnName(), Part::pimItemIdFullColumnName() );
+    partQuery.addJoin( QueryBuilder::InnerJoin, PartType::tableName(), Part::partTypeIdFullColumnName(), PartType::idFullColumnName() );
     partQuery.addColumn( PimItem::idFullColumnName() );
-    partQuery.addColumn( Part::nameFullColumnName() );
+    partQuery.addColumn( PartType::nsFullColumnName() );
+    partQuery.addColumn( PartType::nameFullColumnName() );
     partQuery.addColumn( Part::dataFullColumnName() );
     partQuery.addColumn( Part::externalFullColumnName() );
 
@@ -99,19 +87,26 @@ QSqlQuery FetchHelper::buildPartQuery( const QVector<QByteArray> &partList, bool
     Query::Condition cond( Query::Or );
     if ( !partList.isEmpty() ) {
       QStringList partNameList;
-      partNameList.reserve( partList.size() );
       Q_FOREACH ( const QByteArray &b, partList ) {
-        partNameList.push_back( QString::fromLatin1( b ) );
+        if ( b.startsWith( "PLD" ) || b.startsWith( "ATR" ) ) {
+          partNameList.push_back( QString::fromLatin1( b ) );
+        }
+      }
+      if ( !partNameList.isEmpty() ) {
+        cond.addCondition( PartTypeHelper::conditionFromFqNames( partNameList ) );
       }
-      cond.addValueCondition( Part::nameFullColumnName(), Query::In, partNameList );
     }
+
     if ( allPayload ) {
-      cond.addValueCondition( QString::fromLatin1( "substr( %1, 1, 4 )" ).arg( Part::nameFullColumnName() ), Query::Equals, QLatin1String( AKONADI_PARAM_PLD ) );
+      cond.addValueCondition( PartType::nsFullColumnName(), Query::Equals, QLatin1String( "PLD" ) );
     }
     if ( allAttrs ) {
-      cond.addValueCondition( QString::fromLatin1( "substr( %1, 1, 4 )" ).arg( Part::nameFullColumnName() ), Query::Equals, QLatin1String( AKONADI_PARAM_ATR ) );
+      cond.addValueCondition( PartType::nsFullColumnName(), Query::Equals, QLatin1String( "ATR" ) );
+    }
+
+    if ( !cond.isEmpty() ) {
+      partQuery.addCondition( cond );
     }
-    partQuery.addCondition( cond );
 
     ItemQueryHelper::scopeToQuery( mScope, mConnection, partQuery );
 
@@ -135,22 +130,22 @@ QSqlQuery FetchHelper::buildItemQuery()
   int column = 0;
   #define ADD_COLUMN(colName, colId) { itemQuery.addColumn( colName ); mItemQueryColumnMap[colId] = column++; }
   ADD_COLUMN( PimItem::idFullColumnName(), ItemQueryPimItemIdColumn );
-  if ( mRemoteIdRequested ) {
+  if ( mFetchScope.remoteIdRequested() ) {
     ADD_COLUMN( PimItem::remoteIdFullColumnName(), ItemQueryPimItemRidColumn )
   }
   ADD_COLUMN( MimeType::nameFullColumnName(), ItemQueryMimeTypeColumn )
   ADD_COLUMN( PimItem::revFullColumnName(), ItemQueryRevColumn )
-  if ( mRemoteRevisionRequested ) {
+  if ( mFetchScope.remoteRevisionRequested() ) {
     ADD_COLUMN( PimItem::remoteRevisionFullColumnName(), ItemQueryRemoteRevisionColumn )
   }
-  if ( mSizeRequested ) {
+  if ( mFetchScope.sizeRequested() ) {
     ADD_COLUMN( PimItem::sizeFullColumnName(), ItemQuerySizeColumn )
   }
-  if ( mMTimeRequested ) {
+  if ( mFetchScope.mTimeRequested() ) {
     ADD_COLUMN( PimItem::datetimeFullColumnName(), ItemQueryDatetimeColumn )
   }
   ADD_COLUMN( PimItem::collectionIdFullColumnName(), ItemQueryCollectionIdColumn )
-  if ( mGidRequested ) {
+  if ( mFetchScope.gidRequested() ) {
     ADD_COLUMN( PimItem::gidFullColumnName(), ItemQueryPimItemGidColumn )
   }
   #undef ADD_COLUMN
@@ -161,8 +156,8 @@ QSqlQuery FetchHelper::buildItemQuery()
     ItemQueryHelper::scopeToQuery( mScope, mConnection, itemQuery );
   }
 
-  if ( mChangedSince.isValid() ) {
-    itemQuery.addValueCondition( PimItem::datetimeFullColumnName(), Query::GreaterOrEqual, mChangedSince.toUTC() );
+  if ( mFetchScope.changedSince().isValid() ) {
+    itemQuery.addValueCondition( PimItem::datetimeFullColumnName(), Query::GreaterOrEqual, mFetchScope.changedSince().toUTC() );
   }
 
   if ( !itemQuery.exec() ) {
@@ -200,20 +195,46 @@ QSqlQuery FetchHelper::buildFlagQuery()
   return flagQuery.query();
 }
 
-bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
-{
-  parseCommandStream();
-  triggerOnDemandFetch();
+enum TagQueryColumns {
+  TagQueryItemIdColumn,
+  TagQueryTagIdColumn,
+};
 
+QSqlQuery FetchHelper::buildTagQuery()
+{
+  QueryBuilder tagQuery( PimItem::tableName() );
+  tagQuery.addJoin( QueryBuilder::InnerJoin, PimItemTagRelation::tableName(),
+                     PimItem::idFullColumnName(), PimItemTagRelation::leftFullColumnName() );
+  tagQuery.addJoin( QueryBuilder::InnerJoin, Tag::tableName(),
+                     Tag::idFullColumnName(), PimItemTagRelation::rightFullColumnName() );
+  tagQuery.addColumn( PimItem::idFullColumnName() );
+  tagQuery.addColumn( Tag::idFullColumnName() );
+  ItemQueryHelper::scopeToQuery( mScope, mConnection, tagQuery );
+  tagQuery.addSortColumn( PimItem::idFullColumnName(), Query::Descending );
+
+  if ( !tagQuery.exec() ) {
+    throw HandlerException( "Unable to retrieve item tags" );
+  }
+
+  tagQuery.query().next();
+
+  return tagQuery.query();
+}
+
+bool FetchHelper::fetchItems( const QByteArray &responseIdentifier )
+{
   // retrieve missing parts
-  if ( !mCacheOnly ) {
+  if ( !mFetchScope.cacheOnly() ) {
+    // trigger a collection sync if configured to do so
+    triggerOnDemandFetch();
+
     // Prepare for a call to ItemRetriever::exec();
     // From a resource perspective the only parts that can be fetched are payloads.
     ItemRetriever retriever( mConnection );
     retriever.setScope( mScope );
-    retriever.setRetrieveParts( mRequestedPayloads );
-    retriever.setRetrieveFullPayload( mFullPayload );
-    if ( !retriever.exec() && !mIgnoreErrors ) { // There we go, retrieve the missing parts from the resource.
+    retriever.setRetrieveParts( mFetchScope.requestedPayloads() );
+    retriever.setRetrieveFullPayload( mFetchScope.fullPayload() );
+    if ( !retriever.exec() && !mFetchScope.ignoreErrors() ) { // There we go, retrieve the missing parts from the resource.
       if ( mConnection->resourceContext().isValid() ) {
         throw HandlerException( QString::fromLatin1( "Unable to fetch item from backend (collection %1, resource %2) : %3" )
                 .arg( mConnection->selectedCollectionId() )
@@ -245,16 +266,22 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
   }
   // build part query if needed
   QSqlQuery partQuery;
-  if ( !mRequestedParts.isEmpty() || mFullPayload || mAllAttrs ) {
-    partQuery = buildPartQuery( mRequestedParts, mFullPayload, mAllAttrs );
+  if ( !mFetchScope.requestedParts().isEmpty() || mFetchScope.fullPayload() || mFetchScope.allAttributes() ) {
+    partQuery = buildPartQuery( mFetchScope.requestedParts(), mFetchScope.fullPayload(), mFetchScope.allAttributes() );
   }
 
   // build flag query if needed
   QSqlQuery flagQuery;
-  if ( mFlagsRequested ) {
+  if ( mFetchScope.flagsRequested() ) {
     flagQuery = buildFlagQuery();
   }
 
+  // build tag query if needed
+  QSqlQuery tagQuery;
+  if ( mFetchScope.tagsRequested() ) {
+    tagQuery = buildTagQuery();
+  }
+
   // build responses
   Response response;
   response.setUntagged();
@@ -265,37 +292,37 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
     QList<QByteArray> attributes;
     attributes.append( AKONADI_PARAM_UID " " + QByteArray::number( pimItemId ) );
     attributes.append( AKONADI_PARAM_REVISION " " + QByteArray::number( pimItemRev ) );
-    if ( mRemoteIdRequested ) {
+    if ( mFetchScope.remoteIdRequested() ) {
       attributes.append( AKONADI_PARAM_REMOTEID " " + ImapParser::quote( Utils::variantToByteArray( extractQueryResult( itemQuery, ItemQueryPimItemRidColumn ) ) ) );
     }
     attributes.append( AKONADI_PARAM_MIMETYPE " " + ImapParser::quote( Utils::variantToByteArray( extractQueryResult( itemQuery, ItemQueryMimeTypeColumn ) ) ) );
     Collection::Id parentCollectionId = extractQueryResult( itemQuery, ItemQueryCollectionIdColumn ).toLongLong();
     attributes.append( AKONADI_PARAM_COLLECTIONID " " + QByteArray::number( parentCollectionId ) );
 
-    if ( mSizeRequested ) {
+    if ( mFetchScope.sizeRequested() ) {
       const qint64 pimItemSize = extractQueryResult( itemQuery, ItemQuerySizeColumn ).toLongLong();
       attributes.append( AKONADI_PARAM_SIZE " " + QByteArray::number( pimItemSize ) );
     }
-    if ( mMTimeRequested ) {
+    if ( mFetchScope.mTimeRequested() ) {
       const QDateTime pimItemDatetime = extractQueryResult( itemQuery, ItemQueryDatetimeColumn ).toDateTime();
       // Date time is always stored in UTC time zone by the server.
       QString datetime = QLocale::c().toString( pimItemDatetime, QLatin1String( "dd-MMM-yyyy hh:mm:ss +0000" ) );
       attributes.append( AKONADI_PARAM_MTIME " " + ImapParser::quote( datetime.toUtf8() ) );
     }
-    if ( mRemoteRevisionRequested ) {
+    if ( mFetchScope.remoteRevisionRequested() ) {
       const QByteArray rrev = Utils::variantToByteArray( extractQueryResult( itemQuery, ItemQueryRemoteRevisionColumn ) );
       if ( !rrev.isEmpty() ) {
         attributes.append( AKONADI_PARAM_REMOTEREVISION " " + ImapParser::quote( rrev ) );
       }
     }
-    if ( mGidRequested ) {
+    if ( mFetchScope.gidRequested() ) {
       const QByteArray gid = Utils::variantToByteArray( itemQuery.value( ItemQueryPimItemGidColumn ) );
       if ( !gid.isEmpty() ) {
         attributes.append( AKONADI_PARAM_GID " " + ImapParser::quote( gid ) );
       }
     }
 
-    if ( mFlagsRequested ) {
+    if ( mFetchScope.flagsRequested() ) {
       QList<QByteArray> flags;
       while ( flagQuery.isValid() ) {
         const qint64 id = flagQuery.value( FlagQueryIdColumn ).toLongLong();
@@ -311,8 +338,27 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
       attributes.append( AKONADI_PARAM_FLAGS " (" + ImapParser::join( flags, " " ) + ')' );
     }
 
-    if ( mAncestorDepth > 0 ) {
-      attributes.append( HandlerHelper::ancestorsToByteArray( mAncestorDepth, ancestorsForItem( parentCollectionId ) ) );
+    if ( mFetchScope.tagsRequested() ) {
+      ImapSet tags;
+      while ( tagQuery.isValid() ) {
+        const qint64 id = tagQuery.value( TagQueryItemIdColumn ).toLongLong();
+        if ( id > pimItemId ) {
+          tagQuery.next();
+          continue;
+        } else if ( id < pimItemId ) {
+          break;
+        }
+        const qint64 tagId = tagQuery.value( TagQueryTagIdColumn ).toLongLong();
+        tags.add( QVector<qint64>() << tagId );
+        tagQuery.next();
+      }
+      if ( !tags.isEmpty() ) {
+        attributes.append( AKONADI_PARAM_TAGS " " + tags.toImapSequenceSet() );
+      }
+    }
+
+    if ( mFetchScope.ancestorDepth() > 0 ) {
+      attributes.append( HandlerHelper::ancestorsToByteArray( mFetchScope.ancestorDepth(), ancestorsForItem( parentCollectionId ) ) );
     }
 
     bool skipItem = false;
@@ -327,31 +373,33 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
       } else if ( id < pimItemId ) {
         break;
       }
-      QByteArray partName = Utils::variantToByteArray( partQuery.value( PartQueryNameColumn ) );
-      QByteArray part = Utils::variantToByteArray( partQuery.value( PartQueryNameColumn ) );
+      const QByteArray partName = Utils::variantToByteArray( partQuery.value( PartQueryTypeNamespaceColumn ) ) + ':' +
+          Utils::variantToByteArray( partQuery.value( PartQueryTypeNameColumn ) );
+      QByteArray part = partName;
       QByteArray data = Utils::variantToByteArray( partQuery.value( PartQueryDataColumn ) );
 
-      if ( mCheckCachedPayloadPartsOnly ) {
+      if ( mFetchScope.checkCachedPayloadPartsOnly() ) {
         if ( !data.isEmpty() ) {
           cachedParts << part;
         }
         partQuery.next();
      } else {
-        if ( mIgnoreErrors && data.isEmpty() ) {
+        if ( mFetchScope.ignoreErrors() && data.isEmpty() ) {
           //We wanted the payload, couldn't get it, and are ignoring errors. Skip the item.
-          akDebug() << id << " no payload";
+          //This is not an error though, it's fine to have empty payload parts (to denote existing but not cached parts)
+          //akDebug() << "item" << id << "has an empty payload part in parttable for part" << partName;
           skipItem = true;
           break;
         }
-        bool partIsExternal = partQuery.value( PartQueryExternalColumn ).toBool();
-        if ( !mExternalPayloadSupported && partIsExternal ) { //external payload not supported by the client, translate the data
+        const bool partIsExternal = partQuery.value( PartQueryExternalColumn ).toBool();
+        if ( !mFetchScope.externalPayloadSupported() && partIsExternal ) { //external payload not supported by the client, translate the data
           data = PartHelper::translateData( data, partIsExternal );
         }
         int version = partQuery.value( PartQueryVersionColumn ).toInt();
         if ( version != 0 ) { // '0' is the default, so don't send it
           part += '[' + QByteArray::number( version ) + ']';
         }
-        if (  mExternalPayloadSupported && partIsExternal ) { // external data and this is supported by the client
+        if (  mFetchScope.externalPayloadSupported() && partIsExternal ) { // external data and this is supported by the client
           part += " [FILE] ";
         }
         if ( data.isNull() ) {
@@ -369,7 +417,7 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
           part += data;
         }
 
-        if ( mRequestedParts.contains( partName ) || mFullPayload || mAllAttrs ) {
+        if ( mFetchScope.requestedParts().contains( partName ) || mFetchScope.fullPayload() || mFetchScope.allAttributes() ) {
           attributes << part;
         }
 
@@ -382,7 +430,7 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
       continue;
     }
 
-    if ( mCheckCachedPayloadPartsOnly ) {
+    if ( mFetchScope.checkCachedPayloadPartsOnly() ) {
       attributes.append( AKONADI_PARAM_CACHEDPARTS " (" + ImapParser::join( cachedParts, " " ) + ')' );
     }
 
@@ -397,7 +445,7 @@ bool FetchHelper::parseStream( const QByteArray &responseIdentifier )
   }
 
   // update atime (only if the payload was actually requested, otherwise a simple resource sync prevents cache clearing)
-  if ( needsAccessTimeUpdate( mRequestedParts ) || mFullPayload ) {
+  if ( needsAccessTimeUpdate( mFetchScope.requestedParts() ) || mFetchScope.fullPayload() ) {
     updateItemAccessTime();
   }
 
@@ -429,7 +477,7 @@ void FetchHelper::updateItemAccessTime()
 
 void FetchHelper::triggerOnDemandFetch()
 {
-  if ( mScope.scope() != Scope::None || mConnection->selectedCollectionId() <= 0 || mCacheOnly ) {
+  if ( mScope.scope() != Scope::None || mConnection->selectedCollectionId() <= 0 || mFetchScope.cacheOnly() ) {
     return;
   }
 
@@ -446,83 +494,12 @@ void FetchHelper::triggerOnDemandFetch()
     return;
   }
 
-  IntervalCheck::instance()->requestCollectionSync( collection );
-}
-
-void FetchHelper::parseCommandStream()
-{
-  Q_ASSERT( mStreamParser );
-
-  // macro vs. attribute list
-  Q_FOREVER {
-    if ( mStreamParser->atCommandEnd() ) {
-      break;
-    }
-    if ( mStreamParser->hasList() ) {
-      parsePartList();
-      break;
-    } else {
-      const QByteArray buffer = mStreamParser->readString();
-      if ( buffer == AKONADI_PARAM_CACHEONLY ) {
-        mCacheOnly = true;
-      } else if ( buffer == AKONADI_PARAM_CHECKCACHEDPARTSONLY ) {
-        mCheckCachedPayloadPartsOnly = true;
-      } else if ( buffer == AKONADI_PARAM_ALLATTRIBUTES ) {
-        mAllAttrs = true;
-      } else if ( buffer == AKONADI_PARAM_EXTERNALPAYLOAD ) {
-        mExternalPayloadSupported = true;
-      } else if ( buffer == AKONADI_PARAM_FULLPAYLOAD ) {
-        mRequestedParts << AKONADI_PARAM_PLD_RFC822; // HACK: temporary workaround until we have support for detecting the availability of the full payload in the server
-        mFullPayload = true;
-      } else if ( buffer == AKONADI_PARAM_ANCESTORS ) {
-        mAncestorDepth = HandlerHelper::parseDepth( mStreamParser->readString() );
-      } else if ( buffer == AKONADI_PARAM_IGNOREERRORS ) {
-        mIgnoreErrors = true;
-      } else if ( buffer == AKONADI_PARAM_CHANGEDSINCE ) {
-        bool ok = false;
-        mChangedSince = QDateTime::fromTime_t( mStreamParser->readNumber( &ok ) );
-        if ( !ok ) {
-          throw HandlerException( "Invalid CHANGEDSINCE timestamp" );
-        }
-      } else {
-        throw HandlerException( "Invalid command argument" );
-      }
-    }
-  }
-}
-
-void FetchHelper::parsePartList()
-{
-  mStreamParser->beginList();
-  while ( !mStreamParser->atListEnd() ) {
-    const QByteArray b = mStreamParser->readString();
-    // filter out non-part attributes we send all the time
-    if ( b == AKONADI_PARAM_REVISION || b == AKONADI_PARAM_UID ) {
-      continue;
-    } else if ( b == AKONADI_PARAM_REMOTEID ) {
-      mRemoteIdRequested = true;
-    } else if ( b == AKONADI_PARAM_FLAGS ) {
-      mFlagsRequested = true;
-    } else if ( b == AKONADI_PARAM_SIZE ) {
-      mSizeRequested = true;
-    } else if ( b == AKONADI_PARAM_MTIME ) {
-      mMTimeRequested = true;
-    } else if ( b == AKONADI_PARAM_REMOTEREVISION ) {
-      mRemoteRevisionRequested = true;
-    } else if ( b == AKONADI_PARAM_GID ) {
-      mGidRequested = true;
-    } else {
-      mRequestedParts.push_back( b );
-      if ( b.startsWith( AKONADI_PARAM_PLD ) ) {
-        mRequestedPayloads.push_back( QString::fromLatin1( b ) );
-      }
-    }
-  }
+  IntervalCheck::self()->requestCollectionSync( collection );
 }
 
 QStack<Collection> FetchHelper::ancestorsForItem( Collection::Id parentColId )
 {
-  if ( mAncestorDepth <= 0 || parentColId == 0 ) {
+  if ( mFetchScope.ancestorDepth() <= 0 || parentColId == 0 ) {
     return QStack<Collection>();
   }
   if ( mAncestorCache.contains( parentColId ) ) {
@@ -531,7 +508,7 @@ QStack<Collection> FetchHelper::ancestorsForItem( Collection::Id parentColId )
 
   QStack<Collection> ancestors;
   Collection col = Collection::retrieveById( parentColId );
-  for ( int i = 0; i < mAncestorDepth; ++i ) {
+  for ( int i = 0; i < mFetchScope.ancestorDepth(); ++i ) {
     if ( !col.isValid() ) {
       break;
     }
