@@ -22,7 +22,7 @@
 
 #include <QtCore/QStack>
 
-#include "scope.h"
+#include "fetchscope.h"
 #include "libs/imapset_p.h"
 #include "storage/countquerybuilder.h"
 #include "storage/datastore.h"
@@ -32,8 +32,11 @@ class FetchHelperTest;
 
 namespace Akonadi {
 
-class AkonadiConnection;
 class ImapSet;
+
+namespace Server {
+
+class Connection;
 class Response;
 
 class FetchHelper : public QObject
@@ -41,14 +44,12 @@ class FetchHelper : public QObject
   Q_OBJECT
 
   public:
-    FetchHelper( AkonadiConnection *connection, const Scope &scope );
+    FetchHelper( Connection *connection, const Scope &scope, const FetchScope &fetchScope );
 
-    void setStreamParser( ImapStreamParser *parser );
-
-    bool parseStream( const QByteArray &responseIdentifier );
+    bool fetchItems( const QByteArray &responseIdentifier );
 
   Q_SIGNALS:
-    void responseAvailable( const Akonadi::Response &response );
+    void responseAvailable( const Akonadi::Server::Response &response );
 
   private:
     enum ItemQueryColumns {
@@ -69,8 +70,7 @@ class FetchHelper : public QObject
     QSqlQuery buildItemQuery();
     QSqlQuery buildPartQuery( const QVector<QByteArray> &partList, bool allPayload, bool allAttrs );
     QSqlQuery buildFlagQuery();
-    void parseCommandStream();
-    void parsePartList();
+    QSqlQuery buildTagQuery();
     QStack<Collection> ancestorsForItem( Collection::Id parentColId );
     static bool needsAccessTimeUpdate( const QVector<QByteArray> &parts );
     QVariant extractQueryResult( const QSqlQuery &query, ItemQueryColumns column ) const;
@@ -78,30 +78,16 @@ class FetchHelper : public QObject
   private:
     ImapStreamParser *mStreamParser;
 
-    AkonadiConnection *mConnection;
-    Scope mScope;
-    QVector<QByteArray> mRequestedParts;
-    QStringList mRequestedPayloads;
+    Connection *mConnection;
     QHash<Collection::Id, QStack<Collection> > mAncestorCache;
-    int mAncestorDepth;
-    bool mCacheOnly;
-    bool mCheckCachedPayloadPartsOnly;
-    bool mFullPayload;
-    bool mAllAttrs;
-    bool mSizeRequested;
-    bool mMTimeRequested;
-    bool mExternalPayloadSupported;
-    bool mRemoteRevisionRequested;
-    bool mIgnoreErrors;
-    bool mFlagsRequested;
-    bool mRemoteIdRequested;
-    bool mGidRequested;
-    QDateTime mChangedSince;
+    Scope mScope;
+    FetchScope mFetchScope;
     int mItemQueryColumnMap[ItemQueryColumnCount];
 
     friend class ::FetchHelperTest;
 };
 
-}
+} // namespace Server
+} // namespace Akonadi
 
 #endif
