@@ -21,7 +21,7 @@
 #include <akdebug.h>
 
 #include "akonadi.h"
-#include "akonadiconnection.h"
+#include "connection.h"
 #include "handlerhelper.h"
 #include "response.h"
 #include "storage/datastore.h"
@@ -31,12 +31,12 @@
 #include "storage/parthelper.h"
 #include "storage/dbconfig.h"
 #include "storage/itemretriever.h"
+#include "storage/parttypehelper.h"
 
 #include "libs/imapparser_p.h"
+#include "libs/protocol_p.h"
 #include "imapstreamparser.h"
 
-#include <libs/protocol_p.h>
-
 #include <QtCore/QStringList>
 #include <QLocale>
 #include <QDebug>
@@ -46,6 +46,7 @@
 #include <functional>
 
 using namespace Akonadi;
+using namespace Akonadi::Server;
 
 static bool payloadChanged( const QSet<QByteArray> &changes )
 {
@@ -113,6 +114,35 @@ bool Store::deleteFlags( const PimItem::List &items, const QList<QByteArray> &fl
   return true;
 }
 
+bool Store::replaceTags( const PimItem::List &item, const ImapSet &tags )
+{
+  const Tag::List tagList = HandlerHelper::resolveTags( tags );
+  if ( !connection()->storageBackend()->setItemsTags( item, tagList ) ) {
+    throw HandlerException( "Store::replaceTags: Unable to set new item tags" );
+  }
+  return true;
+}
+
+bool Store::addTags( const PimItem::List &items, const ImapSet &tags, bool &tagsChanged )
+{
+  const Tag::List tagList = HandlerHelper::resolveTags( tags );
+  if ( !connection()->storageBackend()->appendItemsTags( items, tagList, tagsChanged ) ) {
+    akDebug() << "Store::addTags: Unable to add new item tags";
+    return false;
+  }
+  return true;
+}
+
+bool Store::deleteTags( const PimItem::List &items, const ImapSet &tags )
+{
+  const Tag::List tagList = HandlerHelper::resolveTags( tags );
+  if ( !connection()->storageBackend()->removeItemsTags( items, tagList ) ) {
+    akDebug() << "Store::deleteTags: Unable to remove item tags";
+    return false;
+  }
+  return true;
+}
+
 bool Store::parseStream()
 {
   parseCommand();
@@ -201,6 +231,27 @@ bool Store::parseStream()
       continue;
     }
 
+    if ( command == AKONADI_PARAM_TAGS ) {
+      bool tagsChanged = true;
+      const ImapSet tags = m_streamParser->readSequenceSet();
+      if ( op == Replace ) {
+        tagsChanged = replaceTags( pimItems, tags );
+      } else if ( op == Add ) {
+        if ( !addTags( pimItems, tags, tagsChanged ) ) {
+          return failureResponse( "Unable to add item tags." );
+        }
+      } else if ( op == Delete ) {
+        if ( !( tagsChanged = deleteTags( pimItems, tags ) ) ) {
+          return failureResponse( "Unable to remove item tags." );
+        }
+      }
+
+      if ( tagsChanged && !changes.contains( AKONADI_PARAM_TAGS ) ) {
+        changes << AKONADI_PARAM_TAGS;
+      }
+      continue;
+    }
+
     // handle commands that can only be applied to one item
     if ( pimItems.size() > 1 ) {
       throw HandlerException( "This Modification can only be applied to a single item" );
@@ -243,7 +294,7 @@ bool Store::parseStream()
     } else if ( command == AKONADI_PARAM_SIZE ) {
       mSize = m_streamParser->readNumber();
       changes << AKONADI_PARAM_SIZE;
-    } else if ( command == "PARTS" ) {
+    } else if ( command == AKONADI_PARAM_PARTS ) {
       const QList<QByteArray> parts = m_streamParser->readParenthesizedList();
       if ( op == Delete ) {
         if ( !store->removeItemParts( item, parts ) ) {
@@ -254,98 +305,14 @@ bool Store::parseStream()
     } else if ( command == AKONADI_CMD_COLLECTION ) {
       throw HandlerException( "Item moving via STORE is deprecated, update your Akonadi client" );
     } else { // parts/attributes
-      // obtain and configure the part object
-      int partVersion = 0;
-      QByteArray partName;
-      ImapParser::splitVersionedKey( command, partName, partVersion );
-
-      SelectQueryBuilder<Part> qb;
-      qb.addValueCondition( Part::pimItemIdColumn(), Query::Equals, item.id() );
-      qb.addValueCondition( Part::nameColumn(), Query::Equals, QString::fromUtf8( partName ) );
-      if ( !qb.exec() ) {
-        return failureResponse( "Unable to check item part existence" );
-      }
-      Part::List result = qb.result();
-      Part part;
-      if ( !result.isEmpty() ) {
-        part = result.first();
-      }
-      part.setName( QString::fromUtf8( partName ) );
-      part.setVersion( partVersion );
-      part.setPimItemId( item.id() );
-
-      QByteArray value;
-      if ( m_streamParser->hasLiteral() ) {
-        const qint64 dataSize = m_streamParser->remainingLiteralSize();
-        if ( partName.startsWith( AKONADI_PARAM_PLD ) ) {
-          partSizes += dataSize;
-        }
-        const bool storeInFile = dataSize > DbConfig::configuredDatabase()->sizeThreshold();
-        //actual case when streaming storage is used: external payload is enabled, data is big enough in a literal
-        if ( storeInFile ) {
-          // use first part as value for the initial insert into / update to the database.
-          // this will give us a proper filename to stream the rest of the parts contents into
-          // NOTE: we have to set the correct size (== dataSize) directly
-          value = m_streamParser->readLiteralPart();
-         // akDebug() << Q_FUNC_INFO << "VALUE in STORE: " << value << value.size() << dataSize;
-
-          if ( part.isValid() ) {
-            PartHelper::update( &part, value, dataSize );
-          } else {
-//             akDebug() << "insert from Store::handleLine";
-            part.setData( value );
-            part.setDatasize( dataSize );
-            if ( !PartHelper::insert( &part ) ) {
-              return failureResponse( "Unable to add item part" );
-            }
-          }
-
-          //the actual streaming code for the remaining parts:
-          // reads from the parser, writes immediately to the file
-          // ### move this entire block to part helper? should be useful for append as well
-          const QString fileName = PartHelper::resolveAbsolutePath( part.data() );
-          QFile file( fileName );
-          if ( file.open( QIODevice::WriteOnly | QIODevice::Append ) ) {
-            while ( !m_streamParser->atLiteralEnd() ) {
-              value = m_streamParser->readLiteralPart();
-              file.write( value ); // ### error handling?
-            }
-            file.close();
-          } else {
-            return failureResponse( "Unable to update item part" );
-          }
-
-          changes << partName;
-          continue;
-        } else { // not store in file
-          //don't write in streaming way as the data goes to the database
-          while ( !m_streamParser->atLiteralEnd() ) {
-            value += m_streamParser->readLiteralPart();
-          }
-        }
-      } else { //not a literal
-        value = m_streamParser->readString();
-        if ( partName.startsWith( AKONADI_PARAM_PLD ) ) {
-          partSizes += value.size();
-        }
-      }
-
-      // only relevant for non-literals or non-external literals
-      const QByteArray origData = PartHelper::translateData( part );
-      if ( origData != value ) {
-        if ( part.isValid() ) {
-          PartHelper::update( &part, value, value.size() );
-        } else {
-//           akDebug() << "insert from Store::handleLine: " << value.left(100);
-          part.setData( value );
-          part.setDatasize( value.size() );
-          if ( !PartHelper::insert( &part ) ) {
-            return failureResponse( "Unable to add item part" );
-          }
-        }
-        changes << partName;
-      }
+     QByteArray partName, error;
+     qint64 partSize;
+     if ( !PartHelper::storeStreamedParts( command, m_streamParser, item, true, partName, partSize, error ) ) {
+       return failureResponse( error );
+     }
 
+     changes << partName;
+     partSizes += partSize;
     } // parts/attribute modification
   }
 
@@ -442,7 +409,7 @@ void Store::parseCommand()
 void Store::sendPimItemResponse( const PimItem &pimItem )
 {
   QList<QByteArray> attrs;
-  attrs.push_back( "REV" );
+  attrs.push_back( AKONADI_PARAM_REVISION );
   attrs.push_back( QByteArray::number( pimItem.rev() ) );
 
   QByteArray result;
